/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/

#include <stdio.h>
#include "main.h"

/* Exported functions ---------------------------------------------------------*/
extern  uint8_t CAN_Send(CAN_TypeDef *CAN,CanTxMsg *Message);

/* Global Variables ---------------------------------------------------------*/
xQueueHandle xQueueCAN2,xQueueLED,xQueueButton;

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/*--------------- Tasks Priority -------------*/
#define REC_TASK_PRIO     ( tskIDLE_PRIORITY + 2 )
#define SEND_TASK_PRIO    ( tskIDLE_PRIORITY + 1 )

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

/* Private function prototypes -----------------------------------------------*/
void LED_Init(void);


/* Private functions ---------------------------------------------------------*/
/**
  * @brief  Check the button and sends its state via queue. It also checks every 1s one CAN Module flag
  * @param  pvParameters not used
  * @retval None
  */
void vTask_ButtonApli(void * pvParameters)
{
	uint8_t i = 0;
	uint8_t error = 0;
	uint8_t previousstate = 1,state = 1;
    CanTxMsg Message;


    Message.StdId=0x012;
	Message.RTR = CAN_RTR_DATA;
	Message.IDE = CAN_ID_STD;
	Message.DLC = 8;

	for(i=0;i<8;i++)
		Message.Data[i] = 0x00;

	for( ;; )
		{

			switch(state)
				{

					case 1:

						i=0;

						while(i < 100)
							{

						//----------------------------------------------
							if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0))
								{
									Message.Data[0]=0xa3;

								}




						//-------------------------------------------------
							xQueueSendToFront( xQueueButton, &Message,0);


							vTaskDelay(100);

							i++;

							}

						if(32 == previousstate)
							state = 1;
						else
							state = previousstate<<1;

						break;

					case 2:

						state = 1;

						previousstate = 2;
						state = 1;

						break;

					case 4:

						error = CAN_GetLastErrorCode(CAN1);
						error = CAN_GetLastErrorCode(CAN2);

						previousstate = 4;
						state = 1;

						break;

					case 8:

						error = CAN_GetReceiveErrorCounter(CAN1);
						error = CAN_GetReceiveErrorCounter(CAN2);

						previousstate = 8;
						state = 1;

						break;

					case 16:

						error = CAN_GetLSBTransmitErrorCounter(CAN1);
						error = CAN_GetLSBTransmitErrorCounter(CAN2);

						previousstate = 16;
						state = 1;

						break;

					case 32:

						error = CAN_GetFlagStatus(CAN1,CAN_FLAG_FF0);
						error = CAN_GetFlagStatus(CAN2,CAN_FLAG_FF1);

						state = 1;
						previousstate = 32;


					default:

					while(1)
						vTaskDelay(100);

				}
		}
}
/**
  * @brief  CAN1 sends Message with the state of the button
  * @param  pvParameters not used
  * @retval None
  */
void vTask_ButtonStatusSend(void * pvParameters)
{
	uint8_t i;
    CanTxMsg Message;

	  Message.RTR = CAN_RTR_DATA;
	  Message.IDE = CAN_ID_STD;
	  Message.DLC = 8;

	  for(i=0;i<8;i++)
			Message.Data[i] = 0x00;

      for( ;; )
      {

    	xQueueReceive( xQueueButton, &( Message), 0 );		//get Button status

    	//----------------------------------------------------------




    	//-------------------------------------------------------------

    	CAN_Send(CAN1,&Message);

        vTaskDelay(5);

      }
}
/**
  * @brief  Switches LED
  * @param  pvParameters not used
  * @retval None
  */
void vTask_LEDApli ( void * pvParameters )
{
	uint8_t i,buttonstate,queue=0,timeout=0,stop=0;
    CanRxMsg RxMessage;
	static portTickType xLastWakeTime;

	xLastWakeTime = xTaskGetTickCount();


    for(;;)
    {

    	queue=xQueueReceive( xQueueLED, &( RxMessage ), 0 );
      	//------------------------------------------------------

    	if(queue==1 && stop!=1)
    	{

    		buttonstate=RxMessage.Data[0];

    	if(buttonstate == 0xa3)
    		{

    			GPIO_SetBits(GPIOD, GPIO_Pin_12);

    		}
      	if(buttonstate == 0)
    		{

      			GPIO_ResetBits(GPIOD, GPIO_Pin_12);

    		}

      	timeout=0;
    	}
    	else
    	{
    		timeout++;

    		if(timeout>8)
    			{
    				GPIO_SetBits(GPIOD, GPIO_Pin_12);
    				stop=1;
    			}
    	}



    	//-------------------------------------------------------------
      	vTaskDelayUntil( &xLastWakeTime,10 );

    }

}

/**
  * @brief  Get received Message and sets it via queue to LEDApli
  * @param  pvParameters not used
  * @retval None
  */
void vTask_CAN2_ButtonStatusReceived ( void * pvParameters )
{
	uint8_t i;
    CanRxMsg RxMessage;

    for(;;)
    {

      	xQueueReceive( xQueueCAN2, &( RxMessage ), portMAX_DELAY );

      	xQueueSendToFront( xQueueLED, &RxMessage, 0 );

    }

}


/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
	CanRxMsg RxMessage;

	SystemInit();

  /*!< At this stage the microcontroller clock setting is already configured to
       168 MHz, this is done through SystemInit() function which is called from
       startup file (startup_stm32f4xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f4xx.c file
     */

  /*Initialize Leds */
  LED_Init();

  /*Configure the Interrupt*/
  InterruptCAN_Init();

  CAN_Initialisierung();

  /* Create Tasks */

  xQueueCAN2 = xQueueCreate( 1, sizeof( RxMessage ) );
  xQueueLED = xQueueCreate( 1, sizeof( RxMessage ) );
  xQueueButton=xQueueCreate( 1, sizeof( RxMessage ) );

  if( xQueueCAN2 == 0 || xQueueLED ==0 || xQueueButton==0 )
  {
  	while(1)
  	;
  }


  //Handles received Messages From CAN2
  xTaskCreate( vTask_CAN2_ButtonStatusReceived ,NULL, 1024, NULL, REC_TASK_PRIO, NULL );

  xTaskCreate( vTask_ButtonStatusSend ,NULL, 1024, NULL, REC_TASK_PRIO, NULL );

  xTaskCreate( vTask_ButtonApli ,NULL, 1024, NULL, REC_TASK_PRIO, NULL );

  //polls the user button and check the CAN status flags
  xTaskCreate( vTask_LEDApli, NULL, 1024, NULL, SEND_TASK_PRIO, NULL);

  /*Enable CAN Interrupt*/
  CAN_ITConfig(CAN2, CAN_IT_FMP0, ENABLE);
 /*Enable CAN Interrupt
  CAN_ITConfig(CAN1, CAN_IT_FMP1, ENABLE);*/

  /* Start scheduler */
  vTaskStartScheduler();

  /* We should never get here as control is now taken by the scheduler */
  for( ;; );
}

/**
  * @brief  Initializes the LEDs resources and the input for the button
  * @param  None
  * @retval None
  */
void LED_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

  /* Initialize Discovery's LEDs */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

  /* Configure PD15 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12|GPIO_Pin_13|  GPIO_Pin_14;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

  /* Configure PA0 input pulldown mode*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;

  GPIO_Init(GPIOA, &GPIO_InitStructure);

}


/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
